//
// AVR Studio 4.14 Bulid 603
// WinAVR-20080610
//
//
// FM-TEA5767-M : test sample for Tiny2313
// PB0(12) : CLK(4)
// PB1(13) : DATA(5)
//
//
// D:\Hidspx>hidspx -rf
// Detected device is ATtiny2313.
//
// Low: 01100100
//      ||||++++-- CKSEL[3:0] VXeNbNI
//      ||++-- SUT[1:0] N
//      |+-- CKOUT (0:PD2ɃVXeNbNo)
//      +-- CKDIV8 NbNl (1:1/1, 0:1/8)
// 
// High:10-11111
//      |||||||+-- RSTDISBL (RESETs 1:L, 0:(PA2))
//      ||||+++-- BODLEVEL[2:0] (111:Off, 110:1.8, 101:2.7, 100:4.3)
//      |||+-- WDTON (WDT 0:펞ON, 1:ʏ)
//      ||+-- SPIEN (1:ISP֎~, 0:ISP) Parallel̂
//      |+-- EESAVE (EEPROM 1:, 0:ێ)
//      +-- DWEN (On-ChipfobO 1:, 0:L)
// 
// Ext: -------1
//             +-- SPMEN (SPM 1:, 0:L)
// 
// Cal: 92 92
// 

#include <avr/io.h>
#include <util/delay.h>

#define FM_FREQUENCY ( 831 )					// NHK 83.1MHz / FM 76.1MHz

#define	SCL_H { DDRB  &= ~_BV(0); PORTB |= _BV(0); }
#define	SCL_L { PORTB &= ~_BV(0); DDRB  |= _BV(0); }

#define	SDA_H { DDRB  &= ~_BV(1); PORTB |= _BV(1); }
#define	SDA_L { PORTB &= ~_BV(1); DDRB  |= _BV(1); }

#define	IN_SDA ( PINB & _BV(1) )

#define WAIT ( 2 )

/*****************************************************************************
*                    TWIX^[gRfVM
*@Fvoid
*  ߒlFvoid
*****************************************************************************/
void TWI_start( void )
{
	SDA_H;
	SCL_H;
	_delay_us( WAIT );

	SDA_L;
	_delay_us( WAIT );
}

/*****************************************************************************
*                    TWIXgbvRfVM
*@Fvoid
*  ߒlFvoid
*****************************************************************************/
void TWI_stop( void )
{
	SCL_H;
	SDA_L;
	_delay_us( WAIT );

	SDA_H;
	_delay_us( WAIT );
}

/*****************************************************************************
*                    \tgEGATWIM(1byte)
*@Fuint8_t data Mf[^
*  ߒlFuint8_t      MXe[^X 0:ACK  !0:NAK
*****************************************************************************/
uint8_t TWI_send( uint8_t data )
{
	uint8_t cnt = 8;
	while ( cnt-- ) {
		SCL_L;
		if ( data & 0x80 ) {
			SDA_H;
		} else {
			SDA_L;
		}
		SCL_H;
		_delay_us( WAIT );

		data <<= 1;
	}

	SCL_L;
	SDA_H;
	_delay_us( WAIT );

	SCL_H;
	_delay_us( WAIT );

	cnt = IN_SDA;

	SCL_L;
	_delay_us( WAIT );

	return ( cnt );
}

/*****************************************************************************
*                    \tgEGATWIM(1byte)
*@Fuint8_t ack 1oCgMɕԂ 0:NAK  1:ACK
*  ߒlFuint8_t     Mf[^
*****************************************************************************/
uint8_t TWI_recv( uint8_t ack )
{
	uint8_t cnt = 8;
	uint8_t dat = 0;

	SDA_H;
	SCL_L;

	while ( cnt-- ) {
		_delay_us( WAIT );
		SCL_H;

		_delay_us( WAIT );
		dat <<= 1;
		if ( IN_SDA ) {
			dat |= 0x01;
		}

		SCL_L;
	}

	if ( ack ) {
		SDA_L;
	}

	SCL_H;
	_delay_us( WAIT );

	SCL_L;
	SDA_H;
	_delay_us( WAIT );

	return ( dat );
}

/*****************************************************************************
*              FMW[փp[^ݒ
*@Fuint8_t *data 5oCg̃p[^̈ւ̃|C^
*  ߒlFuint8_t 0:OK !0:NG
*****************************************************************************/
uint8_t sendTEA( uint8_t *data )
{
	uint8_t sts = 0;

	TWI_start();

	if ( TWI_send( 0xc0 ) == 0x00 ) {
		uint8_t i = 5;
		while ( i-- ) {
			if ( TWI_send( *data++ ) != 0 ) {
				sts = 5 - i;
			}
		}
	} else {
		sts = 0xff;
	}

	TWI_stop();

	return ( sts );
}

/*****************************************************************************
*              FMW[̃p[^擾
*@Fuint8_t *data 5oCg̃p[^i[̈ւ̃|C^
*  ߒlFuint8_t 0:OK !0:NG
*****************************************************************************/
uint8_t recvTEA( uint8_t *data )
{
	uint8_t sts = 0;

	TWI_start();

	if ( TWI_send( 0xc1 ) == 0x00 ) {
		uint8_t i = 5;
		while ( i-- ) {
			*(data++) = TWI_recv( 1 );
		}
	} else {
		sts = 0xff;
	}

	TWI_stop();

	return ( sts );
}

/*****************************************************************************
*        FMgPLL̒lɕϊ(32bit̍őlɂ107.1MHz܂)
*@Fuint16_t f g10{l( :83.1MHz = 831 )
*  ߒlFuint16_t PLLl
*****************************************************************************/
uint16_t calcPLLvalue( uint16_t f )
{
	return ( (uint16_t)( ( ( ( ( 4UL * ( ( (uint32_t)f * 100UL ) + 225UL ) ) * 10000UL ) / 32768UL ) + 5UL ) / 10UL ) );
}

/*****************************************************************************
*                          C֐
*@Fvoid
*  ߒlFint
*****************************************************************************/
int main( void )
{
	uint8_t sd[5];

	DDRB  = 0b11111100;
	PORTB = 0b00000011;

	DDRD  = 0b11111111;
	PORTD = 0b00000000;

	uint8_t i = 50;
	while ( i-- ) {
		_delay_ms( 1 );
	}

	uint16_t pll = calcPLLvalue( FM_FREQUENCY );
	sd[0] = 0b00000000 | (uint8_t)( ( pll & 0x3f00 ) >> 8 );
	sd[1] = (uint8_t)( pll & 0x00ff );
	sd[2] = 0b00010000;
	sd[3] = 0b00111110;
	sd[4] = 0b00000000;
	sendTEA( sd );

	for ( ;; ) {
		asm("sleep");
	}

	return ( 0 );
}
